----------------------------------------------------------------------------------
-- ECE 3401 2025: PA3 Sparse Matrix Vector Multiplication (SpMV)
-- University of Connecticut
----------------------------------------------------------------------------------
library IEEE;
library work;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.array_pkg.all; --We are using our defined arrays

entity pa3 is
port(
    --State machine input signals
    R_PTR       : in rowp;
    C_PTR       : in nnzs;
    VAL         : in nnzs;
    VEC         : in rows;
    NUM_ROWS    : in integer;
	clk, reset  : in std_logic;
	start       : in std_logic;
	
	--Initialized State machine output signals
    OUTPUT      : out rows; 
    ready       : out std_logic
);
end pa3;

architecture Behavioral of pa3 is

    --Declaration of states
    type state_type is (IDLE, FETCH_ROW, FETCH_NNZ, EXECUTE, UPDATE, RDY);
    signal state   : state_type := IDLE; 

    --Indices for control 
    signal i, s, e, cid : integer := 0;
    
    --Internal partial products accumulator 
    signal tmp    : integer := 0;
    
    --Internal arrays are zeroed initially
    signal r_ptr_int : rowp := (others => 0);
    signal c_ptr_int : nnzs := (others => 0);
    signal val_int : nnzs := (others => 0);
    signal vec_int : rows := (others => 0);
    signal output_int : rows := (others => 0);

    -- Internal signal for number of rows in matrix for SpMV
    signal num_rows_int : integer;
    
begin 

    process( clk )
    begin         
        if clk'event and clk = '1' then
        
            --Check for synchronous reset
            if (reset = '1') then 
                state <= IDLE;
                ready <= '0';
                
                --Load arrays from test bench to design
                r_ptr_int <= R_PTR;
                c_ptr_int <= C_PTR;
                val_int <= VAL;
                vec_int <= VEC;

                --Load the number of rows for SpMV from test bench to design
                num_rows_int <= NUM_ROWS;
                                
                --Reset the internal output signal
                output_int <= (others => 0);
                         
            -- else .. Complete the remaining state machine design as described in the PA3 handout
            
            end if;
        end if;
    end process;
end Behavioral;