----------------------------------------------------------------------------------
-- ECE 3401 2025: PA3 Test Bench
-- University of Connecticut
----------------------------------------------------------------------------------
library IEEE;
library work;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.array_pkg.all; --We are using our defined arrays

entity smtest IS
end smtest;
 
architecture behavior OF smtest IS 
 
    -- Component Declaration for the Unit Under Test (UUT)

    component pa3
    port(
	--Input signals
        R_PTR        : in rowp;
        C_PTR        : in nnzs;
        VAL          : in nnzs;
        VEC          : in rows;
        NUM_ROWS     : in integer;
        clk, reset   : in std_logic;
        start        : in std_logic; 
		
		--Output signals
		OUTPUT       : out rows;
		ready        : out std_logic
	);
    end component;
 	
   -- Clock period definitions
   constant clk_period : time := 10 ns;

   -- Number of Rows to process fo SPMV
   signal NUM_ROWS : integer := 0;

   --Test bench matrices for row pointer, column pointer, value, vector, and output arrays
   signal R_PTR : rowp := (others => 0);
   signal C_PTR : nnzs := (others => 0);
   signal VAL : nnzs := (others => 0);
   signal VEC : rows := (others => 0);
   signal OUTPUT : rows := (others => 0);
                         
   --This is our expected output of A*x
   --constant compOut : rows := (2, 0, 2, 4); 
   signal compOut : rows := (others => 0);
   
   signal start      : std_logic;
   signal clk, reset : std_logic;
   signal ready      : std_logic;
   
begin
	-- Instantiate the Unit Under Test (UUT)
   uut: pa3 PORT MAP (
          R_PTR => R_PTR,
          C_PTR => C_PTR,
          VAL => VAL,
          VEC => VEC,
          NUM_ROWS => NUM_ROWS,
          start => start,
          reset => reset,
          clk => clk,
          OUTPUT => OUTPUT,
          ready => ready
        );
   
   clk_process: process
   begin
		clk <= '1';
		wait for clk_period/2;
		clk <= '0';
		wait for clk_period/2;
   end process;
   
    -- Stimulus process
    stim_proc: process
    begin 
    
    ----------------------
    -- 1st input
    ----------------------
    --Set the input arrays
    R_PTR <= (0, 2, 2, 4, 5, 5, 8, 9, 10);
    C_PTR <= (0, 2, 4, 7, 3, 1, 5, 6, 0, 2);
    VAL   <= (1, 1, 1, 1, 1, 1, 1, 1, 1, 1);
    VEC   <= (1, 1, 1, 1, 1, 1, 1, 1);
    compOut <= (2, 0, 2, 1, 0, 3, 1, 1); 

    NUM_ROWS <= 8;

    --Reset system to load arrays into the state machine
    reset <= '1';
    wait for clk_period;
    reset <= '0';
    wait for clk_period;
                                         
    --Starting the SM by asserting the start bit
    start <= '1'; 
    wait for clk_period;
    start <= '0';
    wait for clk_period;
    
    --Wait for "ready" to change, then capture the output 
    wait on ready;
    wait for 0 ns;
    
    --For loop to compare the outputs
    row_loop1: for i in 0 to (NUM_ROWS-1) loop
        assert (OUTPUT(i) = compOut(i)) report "Error, one of the elements is incorrect" severity FAILURE;
    end loop row_loop1;
    report("Everything looks good!");
    
    --Show output for a cycle
    wait for clk_period;

    ----------------------
    -- 2nd input
    ----------------------
    --Set the input arrays
    R_PTR <= (0, 2, 2, 4, 8, -1, -1, -1, -1);
    C_PTR <= (0, 2, 2, 3, 0, 1, 2, 3, -1, -1);
    VAL   <= (1, 1, 1, 1, 1, 1, 1, 1, -1, -1);
    VEC   <= (1, 1, 1, 1, 1, 1, 1, 1);
    compOut <= (2, 0, 2, 4, -1, -1, -1, -1); 

    NUM_ROWS <= 4;

    --Reset system to load arrays into the state machine
    reset <= '1';
    wait for clk_period;
    reset <= '0';
    wait for clk_period;
                                         
    --Starting the SM by asserting the start bit
    start <= '1'; 
    wait for clk_period;
    start <= '0';
    wait for clk_period;
    
    --Wait for "ready" to change, then capture the output 
    wait on ready;
    wait for 0 ns;
    
    --For loop to compare the outputs
    row_loop2: for i in 0 to (NUM_ROWS-1) loop
        assert (OUTPUT(i) = compOut(i)) report "Error, one of the elements is incorrect" severity FAILURE;
    end loop row_loop2;
    report("Everything looks good!");
    
    --Show output for a cycle
    wait for clk_period;

    ----------------------
    -- 3rd input
    ----------------------
    --Set the input arrays
    R_PTR <= (0, 3, 4, 4, 8, -1, -1, -1, -1);
    C_PTR <= (0, 2, 3, 0, 0, 1, 2, 3, -1, -1);
    VAL   <= (1, 1, 1, 1, 1, 1, 1, 1, -1, -1);
    VEC   <= (1, 1, 1, 1, 1, 1, 1, 1);
    compOut <= (3, 1, 0, 4, -1, -1, -1, -1); 

    NUM_ROWS <= 4;

    --Reset system to load arrays into the state machine
    reset <= '1';
    wait for clk_period;
    reset <= '0';
    wait for clk_period;
                                         
    --Starting the SM by asserting the start bit
    start <= '1'; 
    wait for clk_period;
    start <= '0';
    wait for clk_period;
    
    --Wait for "ready" to change, then capture the output 
    wait on ready;
    wait for 0 ns;
    
    --For loop to compare the outputs
    row_loop3: for i in 0 to (NUM_ROWS-1) loop
        assert (OUTPUT(i) = compOut(i)) report "Error, one of the elements is incorrect" severity FAILURE;
    end loop row_loop3;
    report("Everything looks good!");
    
    --Show output for a cycle
    wait for clk_period;

    ----------------------
    -- 4th input
    ----------------------
    --Set the input arrays
    R_PTR <= (0, 3, 5, -1, -1, -1, -1, -1, -1);
    C_PTR <= (0, 2, 3, 0, 1, -1, -1, -1, -1, -1);
    VAL   <= (1, 1, 1, 1, 1, -1, -1, -1, -1, -1);
    VEC   <= (1, 1, 1, 1, 1, 1, 1, 1);
    compOut <= (3, 2, -1, -1, -1, -1, -1, -1); 

    NUM_ROWS <= 2;

    --Reset system to load arrays into the state machine
    reset <= '1';
    wait for clk_period;
    reset <= '0';
    wait for clk_period;
                                         
    --Starting the SM by asserting the start bit
    start <= '1'; 
    wait for clk_period;
    start <= '0';
    wait for clk_period;
    
    --Wait for "ready" to change, then capture the output 
    wait on ready;
    wait for 0 ns;
    
    --For loop to compare the outputs
    row_loop4: for i in 0 to (NUM_ROWS-1) loop
        assert (OUTPUT(i) = compOut(i)) report "Error, one of the elements is incorrect" severity FAILURE;
    end loop row_loop4;
    report("Everything looks good!");
    
    --Show output for a cycle
    wait for clk_period;


    
   end process;
   
END;
